﻿using Kivii.Linq;
using Kivii.Messaging;
using Kivii.Sample.ImageUploader.Entities;
using Kivii.Video;
using Kivii.Video.DirectShow;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Kivii.Sample.ImageUploader
{
    public partial class FrmMain : Form
    {
        private JsonServiceClient _client = null;

        private FilterInfoCollection videoDevices;
        private VideoCaptureDevice videoDevice;
        private VideoCapabilities[] videoCapabilities;
        private int currentCameraIndex = 0;
        private int cameraQuantity = 0;//相机数量
        private List<Photo> photos = new List<Photo>();

        public FrmMain()
        {
            InitializeComponent();
        }


        #region 界面控件默认值管理
        private void loadDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null) continue;
                    ctl.Text = setting.ControlValue;
                }
            }
        }

        private void setDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null)
                    {
                        setting = new UiSetting();
                        setting.FormName = this.Name;
                        setting.ControlName = ctl.Name;
                        setting.ControlValue = ctl.Text;
                        conn.Insert(setting);
                    }
                    else
                    {
                        setting.ControlValue = ctl.Text;
                        conn.Update(setting);
                    }
                }
            }
        }

        private void initFormCtlEnable(bool notLogin = true)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => initFormCtlEnable(notLogin)));
                return;
            }
            btnLogin.Enabled = notLogin;
            tbxServiceUrl.Enabled = notLogin;
            tbxUserName.Enabled = notLogin;
            tbxPassword.Enabled = notLogin;
            cbxResolution.Enabled = !notLogin;
            btnSwitch.Enabled = !notLogin;
            tbxReportId.Enabled = !notLogin;
            btnReportId.Enabled = !notLogin;
            btnShot.Enabled = !notLogin;
            btnDelete.Enabled = !notLogin;
            btnUpload.Enabled = !notLogin;
            btnLogout.Enabled = !notLogin;

            if (notLogin)
            {
                cbxResolution.SelectedIndex = -1;
                //cbxResolution.Items.Clear();
                currentCameraIndex = 0;
                cameraQuantity = 0;
                gbCamView.Text = "相机视图";
                tbxReportId.Text = string.Empty;
                lbCurrentReportId.Text = "__________";
                lbCurrentSampleName.Text = "__________";
                lbCurrentBrand.Text = "__________";
                lbCurrentBatchNumber.Text = "__________";
                lbCurrentSampleQuantity.Text = "__________";
                ptbPhotoDisplay.Image = null;
                flpPhotosTaken.Controls.Clear();
            }
        }
        #endregion

        #region 相机控件调用
        private void initCameras()
        {
            try
            {
                // 初始化摄像头
                videoDevices = new FilterInfoCollection(FilterCategory.VideoInputDevice);
                if (videoDevices.Count != 0)
                {
                    // add all devices to combo
                    cameraQuantity = videoDevices.Count;
                    videoDevice = new VideoCaptureDevice(videoDevices[currentCameraIndex].MonikerString);
                    gbCamView.Text = videoDevices[currentCameraIndex].Name;
                    enumeratedSupportedFrameSizes(videoDevice);
                }
                else
                {
                    MessageBox.Show("没有找到视频采集设备");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("无法初始化摄像头：" + ex.Message);
            }
        }

        private void switchCamera()
        {
            if (videoDevice.IsRunning)
            {
                Disconnect();
            }
            if (currentCameraIndex + 1 > cameraQuantity - 1) currentCameraIndex = 0;
            else currentCameraIndex = currentCameraIndex + 1;
            videoDevice = new VideoCaptureDevice(videoDevices[currentCameraIndex].MonikerString);
            gbCamView.Text = videoDevices[currentCameraIndex].Name;
            enumeratedSupportedFrameSizes(videoDevice);
            // 默认选择第一个分辨率
            cbxResolution.SelectedIndex = 0;
        }

        /// <summary>
        /// 列出摄像头支持的分辨率
        /// </summary>
        /// <param name="videoDevice"></param>
        private void enumeratedSupportedFrameSizes(VideoCaptureDevice videoDevice)
        {
            this.Cursor = Cursors.WaitCursor;
            cbxResolution.Items.Clear();
            try
            {
                // 获取摄像头支持的分辨率列表
                VideoCapabilities[] capabilities = videoDevice.VideoCapabilities;

                // 遍历分辨率列表，并添加到下拉框中
                foreach (VideoCapabilities capability in capabilities)
                {
                    cbxResolution.Items.Add(string.Format("{0} x {1}",
                        capability.FrameSize.Width, capability.FrameSize.Height));
                }
                // 如果没有支持的分辨率，则添加 "Not supported"
                if (cbxResolution.Items.Count == 0)
                {
                    cbxResolution.Items.Add("Not supported");
                }
                //// 默认选择第一个分辨率
                //cbxResolution.SelectedIndex = 0;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        private void Connect(int selectedIndex = 0)
        {
            if (videoDevice == null) return;
            if ((videoCapabilities != null) && (videoCapabilities.Length != 0))
            {
                videoDevice.VideoResolution = videoCapabilities[selectedIndex];
            }
            vspCamView.VideoSource = videoDevice;
            vspCamView.Start();
        }

        private void Disconnect()
        {
            if (vspCamView.VideoSource != null)
            {
                // stop video device
                vspCamView.WaitForStop();
                vspCamView.VideoSource = null;
            }
        }
        #endregion

        private void FrmMain_Load(object sender, EventArgs e)
        {
            initFormCtlEnable(true);
            loadDefault(tbxServiceUrl, tbxUserName, tbxPassword);
            initCameras();
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            if (_client != null)
            {
                MessageBox.Show("登陆前先退出原连接");
                return;
            }
            btnLogin.Enabled = false;

            _client = new JsonServiceClient(tbxServiceUrl.Text);
            var auth = new Authenticate { UserName = tbxUserName.Text, Password = tbxPassword.Text, provider = "Kivii" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                initFormCtlEnable(false);
                setDefault(tbxServiceUrl, tbxUserName, tbxPassword);
                Thread.Sleep(100);
                if (videoDevice == null) initCameras();
                // 默认选择第一个分辨率
                cbxResolution.SelectedIndex = 0;
            }, true);
            task.Error(err =>
            {
                initFormCtlEnable();
                _client.Dispose();
                _client = null;
                var msg = err.GetResponseStatus().Message;
                MessageBox.Show(msg, "Login Error");
            }, true);
        }

        private void btnLogout_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            btnLogout.Enabled = false;

            var auth = new Authenticate { provider = "Logout" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                _client.Dispose();
                _client = null;
                Thread.Sleep(100);
                initFormCtlEnable();
                Disconnect();
            });
            task.Error(err =>
            {
                btnLogout.Enabled = true;
                MessageBox.Show(err.Message, "Logout Error");
            });
        }

        private void cbxResolution_SelectedIndexChanged(object sender, EventArgs e)
        {
            Disconnect();
            if(cbxResolution.SelectedIndex>=0) Connect(cbxResolution.SelectedIndex);
        }

        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            Disconnect();
        }

        private void btnSwitch_Click(object sender, EventArgs e)
        {
            switchCamera();
        }

        private void btnShot_Click(object sender, EventArgs e)
        {
            try
            {
                if (videoDevice != null && vspCamView.IsRunning)
                {
                    // 捕获当前帧
                    Bitmap bitmap = vspCamView.GetCurrentVideoFrame();
                    ptbPhotoDisplay.Image = bitmap;

                    // 创建图片 PictureBox 控件
                    PictureBox pictureBox = new PictureBox();
                    pictureBox.SizeMode = PictureBoxSizeMode.Zoom;
                    pictureBox.Dock = DockStyle.Fill;
                    pictureBox.Image = bitmap;

                    // 添加图片点击动作
                    pictureBox.Click += PictureBox_Click;

                    // 创建图片名称的 Label 控件
                    Label nameLabel = new Label();
                    String name = DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss");
                    nameLabel.Text = name;
                    nameLabel.Dock = DockStyle.Fill;
                    nameLabel.TextAlign = ContentAlignment.MiddleCenter;

                    // 创建 SplitContainer 控件
                    SplitContainer splitContainer = new SplitContainer();
                    splitContainer.Size = new Size(210, 140);
                    splitContainer.Orientation = Orientation.Horizontal;
                    splitContainer.SplitterWidth = 1;
                    splitContainer.SplitterDistance = 100;
                    splitContainer.Panel1.Padding = new Padding(0, 10, 0, 0);
                    splitContainer.Panel1.Controls.Add(pictureBox);
                    splitContainer.Panel2.Controls.Add(nameLabel);

                    // 将 SplitContainer 添加到 flpPhotosTaken 中
                    flpPhotosTaken.Controls.Add(splitContainer);
                    // 改变排序，将新增的添加到顶部
                    flpPhotosTaken.Controls.SetChildIndex(splitContainer, 0);

                    // 将拍摄的照片对象添加到集合中
                    photos.Add(new Photo(bitmap, name));

                    // 遍历 flpPhotosTaken 中的控件，将之前选中的 SplitContainer 边框改为默认
                    foreach (Control control in flpPhotosTaken.Controls)
                    {
                        if (control is SplitContainer controlContainer)
                        {
                            controlContainer.BorderStyle = BorderStyle.None;
                        }
                    }
                    // 为新添加的 SplitContainer 添加边框
                    splitContainer.BorderStyle = BorderStyle.FixedSingle;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("无法拍照" + ex.Message);
            }
        }

        private void PictureBox_Click(object sender, EventArgs e)
        {
            // 获取点击的 PictureBox 中的图片
            PictureBox pictureBox = (PictureBox)sender;

            // 遍历 flpPhotosTaken 中的控件，找到对应的 SplitContainer，并为其设置边框样式
            foreach (Control control in flpPhotosTaken.Controls)
            {
                if (control is SplitContainer splitContainer)
                {
                    // 如果 SplitContainer 包含点击的 PictureBox，则设置其边框,否则设置为默认
                    if (splitContainer.Panel1.Controls.Count > 0 && splitContainer.Panel1.Controls[0] == pictureBox)
                    {
                        splitContainer.BorderStyle = BorderStyle.FixedSingle;
                    }
                    else
                    {
                        splitContainer.BorderStyle = BorderStyle.None;
                    }
                }
            }
            // 在界面中的 PictureBox 中显示该图片
            ptbPhotoDisplay.Image = (Bitmap)pictureBox.Image;
        }
    }
}
